/*
	File:		FileLogging.h

	Contains:	File logging engine for OTStreamLogViewer.

	Written by: Quinn "The Eskimo!"	

	Copyright:	Copyright  1998-1999 by Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):
				7/23/1999	Karl Groethe	Updated for Metrowerks Codewarror Pro 2.1
				

*/

/////////////////////////////////////////////////////////////////////
// Pick up LogEntryPtr definition.

#include "LogEngine.h"

/////////////////////////////////////////////////////////////////////

extern OSStatus StartFileLogging(void);
	// Tells the file log module that we're about to
	// start logging records to the file using RecordLogEntryToFile.
	// Returns an error if the file logging module could not
	// start up, in which case the client should not call
	// StopFileLogging.

// The following two routines both use the same assembly
// buffer. If you call LogEntryToCString, you must copy the
// results out of the buffer before you call either of
// these routines again.

extern char *LogEntryToCString(LogEntry* thisEntry);
	// Converts a log entry to a string.  The log module
	// has to do this anyway, so we export the function
	// for use by the clipboard code.
	
extern void RecordLogEntryToFile(LogEntry* thisEntry);
	// Record a log entry to the log file.

extern void FileLoggingIdle(void);
	// This routine is called periodically by the client
	// to flush out any pending log entries.
	
extern Boolean FileLoggingActive(void);
	// Returns whether a log file is currently open or not.

extern void StopFileLogging(void);
	// Called by the client to shut down the file logging
	// module.
	